#!/bin/bash

set -e
shopt -s extglob

DEST="$1"
TARGET="$2"

function usage {
    echo "Usage: $0 <DEST> [TARGET]"
    echo "where:"
    echo "    <DEST> base install directory"
    echo "    <TARGET> directory to install to (default: canrestbus)"
    echo
    echo "If \$DEST is a matlab directory (\$DEST/rtw/c exists),"
    echo "this command installs etherlab to \$DEST/rtw/c/\$TARGET,"
    echo "otherwise canrestbus is installed to \$DEST/\$TARGET."
    exit 0
}

if [ -z "${DEST}" ]; then
    usage
fi

if [ -z "${TARGET}" ]; then
    TARGET=canrestbus
fi

if test -d "${DEST}/rtw/c"; then
    DST="${DEST}/rtw/c/${TARGET}"
else
    DST="${DEST}/${TARGET}"
fi

if [ ! -d "${DST}" ]; then
    echo Creating folder ${DST}...
    mkdir -p "${DST}"
    mkdir -p "${DST}/sfun"
    mkdir -p "${DST}/mex"
fi

cp -av ./sfun/*.c ${DST}/sfun/
cp -av ./sfun/*.h ${DST}/sfun/
cp -av ./sfun/*.tlc ${DST}/sfun/
cp -av ./sfun/*.m ${DST}/sfun/
cp -av ./mex/*.m ${DST}/mex/
cp -av ./mex/*.c ${DST}/mex/
cp -av *.mdl ${DST}/
cp -av *.m ${DST}/

echo "Files copied to ${DST}"
echo "Now start matlab and execute 'run ${DST}/setup_canrestbus.m'"
