/*  Copyright (C) 2018 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#define S_FUNCTION_NAME  sfun_canrestbus_debug
#define S_FUNCTION_LEVEL 2

#include <inttypes.h>

#include "simstruc.h"

#include <restbus/cs_signal.h>


/*Defines*/

/*Parameter*/

#define PARAM_COUNT 1

#define PARAM_DEBUG 0

#define PARAM_ARG(PARAM)  (ssGetSFcnParam(S, PARAM))

/*====================*
 * S-function methods 
 *====================*/

/* Function: mdlInitializeSizes =============================================
 * Abstract:
 *    The sizes information is used by Simulink to determine the S-function
 *    block's characteristics (number of inputs, outputs, states, etc.).
 */
static void mdlInitializeSizes(SimStruct *S)
{
   int_T i;
   uint8_t dbl;

   ssSetNumSFcnParams(S, PARAM_COUNT);  /* Number of expected parameters */
   if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
     /* Return if number of expected != number of actual parameters */
     ssSetErrorStatus(S, "Parameter Failure");
     return;
   }
   
   for( i = 0; i < PARAM_COUNT; i++)
     ssSetSFcnParamTunable(S, i, SS_PRM_NOT_TUNABLE);
   
   
   if(mxIsEmpty(PARAM_ARG(PARAM_DEBUG))){
     ssSetErrorStatus(S, "No valid Debug level spezified");
     return;
   }   
   
   dbl = (uint8_t)mxGetScalar(PARAM_ARG(PARAM_DEBUG));
   /*ssPrintf("Debuglevel %i\n",dbl);*/
   
   switch(dbl){
   case 1: /*SIGNAL_DEBUG_LEVEL_NONE*/
     cs_signal_debuglevel(CS_DEBUG_LEVEL_NONE);
     break;
   case 2: /*SIGNAL_DEBUG_LEVEL_DEBUG*/
     cs_signal_debuglevel(CS_DEBUG_LEVEL_DEBUG);
     break;
   case 3: /*SIGNAL_DEBUG_LEVEL_VERBOSE*/
     cs_signal_debuglevel(CS_DEBUG_LEVEL_VERBOSE);
     break;
   default:
     ssSetErrorStatus(S, "No valid Debug level value spezified");
     return;
   }
   
   ssSetNumContStates(S, 0);
   ssSetNumDiscStates(S, 0);
   ssSetNumSampleTimes(S, 1);
   ssSetNumRWork(S, 0);
   ssSetNumIWork(S, 0);
   ssSetNumPWork(S, 0);
   ssSetNumModes(S, 0);
   ssSetNumNonsampledZCs(S, 0);
   
   /* use generated code in Accelerator Mode */
   /*ssSetOptions(S, SS_OPTION_USE_TLC_WITH_ACCELERATOR
		| SS_OPTION_CALL_TERMINATE_ON_EXIT);
   */
   ssSetOptions(S,
	 SS_OPTION_WORKS_WITH_CODE_REUSE
	 | SS_OPTION_RUNTIME_EXCEPTION_FREE_CODE
	 | SS_OPTION_CALL_TERMINATE_ON_EXIT);
}
  
/* Function: mdlInitializeSampleTimes =======================================
 * Abstract:
 *    This function is used to specify the sample time(s) for your
 *    S-function. You must register the same number of sample times as
 *    specified in ssSetNumSampleTimes.
 */
static void mdlInitializeSampleTimes(SimStruct *S)
{

}

#define MDL_START                      /* Change to #undef to remove function */
#if defined(MDL_START)
/* Function: mdlStart ==========================================================
 * Abstract:
 *      Here we cache the state (true/false) of the XDATAEVENLYSPACED parameter.
 *      We do this primarily to illustrate how to "cache" parameter values (or
 *      information which is computed from parameter values) which do not change
 *      for the duration of the simulation (or in the generated code). In this
 *      case, rather than repeated calls to mxGetPr, we save the state once.
 *      This results in a slight increase in performance.
 */
static void mdlStart(SimStruct *S)
{

}
#endif /*  MDL_START */



#define MDL_UPDATE
static void mdlUpdate(SimStruct *S, int_T tid)
{
    /** Needed for Matlab versions from 2007b, otherwise no update code is
     * generated.
     */
}

/* Function: mdlOutputs =====================================================
 * Abstract:
 *    In this function, you compute the outputs of your S-function
 *    block. Generally outputs are placed in the output vector, ssGetY(S).
 */
static void mdlOutputs(SimStruct *S, int_T tid)
{

}

/* Function: mdlTerminate ===================================================
 * Abstract:
 *    In this function, you should perform any actions that are necessary
 *    at the termination of a simulation.  For example, if memory was
 *    allocated in mdlStart, this is the place to free it.
 */
static void mdlTerminate(SimStruct *S)
{

}


#undef MDL_RTW  /* Change to #undef to remove function */
#if defined(MDL_RTW) && (defined(MATLAB_MEX_FILE) || defined(NRT))
static void mdlRTW(SimStruct *S){

}
#endif

/*======================================================*
 * See sfuntmpl_doc.c for the optional S-function methods *
 *======================================================*/

/*=============================*
 * Required S-function trailer *
 *=============================*/

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
